# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from django.db.models import Q
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from ckeditor.fields import RichTextField

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel
from jmb.core import db
from jmb.i18n.models import Language
from jmb.address.models import Address
from jmb.payment.models import PaymentForm
from jmb.office.models import Office
from jmb.vat.models import Vat
from jmb.currency.models import Currency
from jmb.carriage.models import Delivery


OrganizationMixin = db.get_mixin('OrganizationOrganizationMixin')
ContactMixin = db.get_mixin('OrganizationContactMixin')

################################
# ANAGRAFICA                   #
################################
class OrganizationAddress (Address):

    """
    Extend `address.Address` with 'organization.Organization`
    MODELS:
    :model: `organization.Organization`
    """
    organization = models.ForeignKey(
        "Organization",
        db_index=True,
        related_name='addresses',
        verbose_name=_("organization"),
    )
    # il nome della società a cui l'indirizzo si riferisce potrebbe essere
    # diversa da 'Organization'
    org_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Organization Name"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("organization address")
        verbose_name_plural = _("organization addresses")


class OrganizationManager(models.Manager):

    def get_query_set(self):
        return super(OrganizationManager, self).get_query_set()


class OwnerOrganizationManager(models.Manager):

    def get_query_set(self):
        qs = super(OwnerOrganizationManager, self).get_query_set()
        return qs.filter(organization_type__category='com', status=1)


class CustomerOrganizationManager(models.Manager):

    def get_query_set(self):
        qs = super(CustomerOrganizationManager, self).get_query_set()
        return qs.filter(organization_type__category='cus', status=1)


class CustomerOwnerOrganizationManager(models.Manager):

    """Mainly used by ticket, where you need to select a customer or the owner"""

    def get_query_set(self):
        qs = super(CustomerOwnerOrganizationManager, self).get_query_set()
        return qs.exclude(organization_type__category='sup').filter(status=1)


class SupplierOrganizationManager(models.Manager):

    def get_query_set(self):
        qs = super(SupplierOrganizationManager, self).get_query_set()
        return qs.filter(organization_type__category='sup', status=1)


class Organization (DateModel, UserModel, StatusModel, OrderedModel, OrganizationMixin):

    """
    Stores organization
    MODELS:
    :model: `organization.Office`
    :model: `organization.OrganizationGroup`
    :model: `organization.OrganizationType`
    :model: `organization.OrganizationForm`
    :model: `organization.Field`
    :model: `payment.PaymentForm`
    :model: `organization.Currency`
    :model: `organization.Delivery`
    :model: `organization.Vat`
    """

    objects = OrganizationManager()
    owners = OwnerOrganizationManager()
    customers_owners = CustomerOwnerOrganizationManager()
    customers = CustomerOrganizationManager()
    suppliers = SupplierOrganizationManager()

    office = models.ForeignKey(
        Office,
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("office"),
    )
    organization_groups = models.ManyToManyField(
        "OrganizationGroup",
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("group"),
    )
    organization_type = models.ForeignKey(
        "OrganizationType",
        db_index=True,
        related_name="organizations",
        verbose_name=_("Organization type"),
    )
    organization_form = models.ForeignKey(
        "OrganizationForm",
        db_index=True,
        related_name="organizations",
        verbose_name=_("Organization form"),
    )
    field = models.ForeignKey(
        "Field",
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("Field"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_('Payment form'),
    )
    currency = models.ForeignKey(
        Currency,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("Currency"),
    )
    delivery = models.ForeignKey(
        Delivery,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("Delivery"),
    )
    vat = models.ForeignKey(
        Vat,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("Vat"),
    )
    code = models.CharField(
        unique=True,
        max_length=15,
        db_index=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    brand = models.CharField(
        max_length=255,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("brand"),
    )
    ssn = models.CharField(
        max_length=16,
        blank=False, null=True,
        db_index=True,
        verbose_name=_("ssn"),
    )
    vat_number = models.CharField(
        max_length=14,
        blank=False, null=True,
        db_index=True,
        verbose_name=_("vat number"),
    )
    url = models.URLField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("url"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("email"),
    )
    phone = models.CharField(
        max_length=20,
        verbose_name=_("phone"),
    )
    phone2 = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("phone2"),
    )
    fax = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("fax"),
    )
    note = RichTextField(
        blank=True, null=True,
        verbose_name=_("note"),
    )

    class Meta:
        unique_together = settings.ORGANIZATION_ORGANIZATION_UNIQUE_TOGETHER
        verbose_name = _("organization")
        verbose_name_plural = _("organizations")
        permissions = (
            ("view_organization", "Can view organizations"),
            ("save_addressbook", "Can save in phone addreebook"),
        )
        ordering = ['name']

    def __unicode__(self):
        return u"%s" % self.name

    def save(self, *args, **kwargs):
        ## Se l'anagrafica è di tipo vettore non possono essere agganciati altri vettori
        if self.organization_type.category == "vec" and self.vectors.all():
            raise ValidationError("Vector can't have Vectors")

        ## modifico il valore del name
        osn = settings.ORGANIZATION_ORGANIZATION_NAME_STYLE
        if osn == "title":
            self.name = self.name.title()
        elif osn == "lower":
            self.name = self.name.lower()
        elif osn == "upper":
            self.name = self.name.upper()

        super(Organization, self).save(*args, **kwargs)

    def get_addresses(self):
        """
        Returns all active addresses
        """
        return self.addresses.filter(status=1)

    def get_first_address(self):
        """
        Returns first active address
        """
        try:
            return self.get_addresses()[0]
        except:
            return None

    def get_shipping_addresses(self):
        """
        Returns a list of active shipping addresses
        """
        addresses = self.get_addresses()
        shipping_addresses = []
        for address in addresses:
            is_shipping_addresses = address.address_scopes.all().filter(is_shipping_address=True)
            if is_shipping_addresses:
                shipping_addresses.append(address)
        return shipping_addresses

    def get_invoice_addresses(self):
        """
        Returns a list of active invoice addresses
        """
        addresses = self.get_addresses()
        invoice_addresses = []
        for address in addresses:
            is_invoice_addresses = address.address_scopes.all().filter(is_invoice_address=True)
            if is_invoice_addresses:
                invoice_addresses.append(address)
        return invoice_addresses

    def default_invoice_address1(self):
        """
        Returns first active invoice address.
        address1 = `address.type + address.name + address.number`
        """
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)
        except:
            return None

    def default_invoice_address2(self):
        """
        Returns first active invoice address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)
        except:
            return None

    def default_invoice_address3(self):
        """
        Returns first active invoice address.
        address3 = `address.country`
        """
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s" % addr.country
        except:
            return None

    def default_shipping_address1(self):
        """
        Returns first active shipping address.
        address1 = `address.type + address.name + address.number`
        """
        try:
            addr = self.get_shipping_addresses()[0]
            return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)
        except:
            return None

    def default_shipping_address2(self):
        """
        Returns first active shipping address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        try:
            addr = self.get_shipping_addresses()[0]
            return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)
        except:
            return None

    def default_shipping_address3(self):
        """
        Returns first active shipping address.
        address3 = `address.country`
        """
        try:
            addr = self.get_shipping_addresses()[0]
            return "%s" % addr.country
        except:
            return None

#    def get_agents(self):
#        """
#        Returns all active agents
#        """
#        return self.agent.filter(status=1)
#
#    def get_commisions(self):
#        """
#        Returns all commisions
#        """
#        return self.commisions.all()
#
#    def get_commision(self):
#        """
#        Returns first commision
#        """
#        try:
#            return self.get_commisions()[0]
#        except:
#            return None

    def get_contacts(self, other_filters={}):
        """
        Returns all active contacts
        """
        contacts = self.contacts.filter(status=1)
        if other_filters:
            contacts = contacts.filter(**other_filters)
        return contacts

    def get_default_contact(self):
        """
        Returns default contact: 'contact.default=True'
        """
        contact_filter = {'default': True}
        contacts = self.get_contacts(other_filters=contact_filter)
        if contacts:
            return contacts[0]
        return None


#    def get_quotation_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_quotation_by_email=True'
#        """
#        contact_filter = {'send_quotation_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#    def get_procedure_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_procedure_by_email=True'
#        """
#        contact_filter = {'send_procedure_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_ddt_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_ddt_by_email=True'
#        """
#        contact_filter = {'send_ddt_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_invoice_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_invoice_by_email=True'
#        """
#        contact_filter = {'send_invoice_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_all_mails_for_procedure(self):
#        """
#        Returns all contact mails with 'send_procedure_by_email=True'
#        """
#        contact_filter = {'send_procedure_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#
#    def get_all_mails_for_ddt(self):
#        """
#        Returns all contact mails with 'send_ddt_by_email=True'
#        """
#        contact_filter = {'send_ddt_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#
#    def get_all_mails_for_invoice(self):
#        """
#        Returns all contact mails with 'send_invoice_by_email=True'
#        """
#        contact_filter = {'send_invoice_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#    def get_all_mails_for_quotation(self):
#        """
#        Returns all contact mails with 'send_quotation_by_email=True'
#        """
#        contact_filter = {'send_quotation_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#    def get_all_mails(self, contact_filter={}):
#        """
#        Returns organization email, contacts emails and agents email.
#        Example:
#        (
#            ('info@thundersystems.it (Organization)', 'info@thundersystems.it'),
#            ('vparenti@thundersystems.it (Contact)', 'vparenti@thundersystems.it'),
#            …
#        )
#        """
#        mails = []
#
#        # Organization email
#        if self.email:
#            label_mail = ugettext("%(email)s (Organization)" % {'email':self.email})
#            choice_mail = (self.email, label_mail)
#            if choice_mail not in mails:
#                mails.append(choice_mail)
#
#        # All contacts emails
#        for c in self.get_contacts(other_filters=contact_filter):
#            if c.email:
#                label_mail = ugettext("%(email)s (Contact)" % {'email':c.email})
#                choice_mail = (c.email, label_mail)
#                if choice_mail not in mails:
#                    mails.append(choice_mail)
#
#        # All agents emails
#        for a in self.get_agents():
#            if a.email:
#                label_mail = ugettext("%(email)s (Agent)" % {'email':a.email})
#                choice_mail = (a.email, label_mail)
#                if choice_mail not in mails:
#                    mails.append(choice_mail)
#
#        return mails
#
#    if "agenda" in settings.INSTALLED_APPS:
#        from django.contrib.contenttypes import generic
#        from agenda.models import Membership
#        memberships = generic.GenericRelation(Membership)
#
#        def get_appointments(self):
#            """
#            Returns all appointments
#            """
#            memberships = self.memberships.all()
#            appointments = {}
#            for m in memberships:
#                if not appointments.has_key(m.appointment.id):
#                    appointments[m.appointment.id] = m.appointment
#            return appointments.values()
#
#    if "commision" in settings.INSTALLED_APPS:
#        def get_balance(self, date_b=None):
#            """
#            restituisce il saldo dell'Anagrafica
#            alla data specificata
#            """
#            cursor = connection.cursor()
#            sql = """
#                SELECT SUM(amount) AS balance
#                FROM commision_commision
#                WHERE organization_id = %s
#                AND operation_date <= %s
#            """
#            if not date_b: date_b = date.today()
#            params = [self.id, date_b.strftime('%Y-%m-%d')]
#            cursor.execute(sql, params)
#            row = cursor.fetchone()
#            if row[0] is None:
#                return 0.00
#            else:
#                return row[0]
#
#    if "commision" in settings.INSTALLED_APPS:
#        def is_insolvent(self, date_b=None):
#            """
#            restituisce True o False se il cliente ha un saldo negativo alla data specificata
#            """
#            if not date_b: date_b = date.today()
#            if self.get_balance(date_b) < 0: return True
#            return False
#
#    if "ddt" in settings.INSTALLED_APPS:
#        def get_ddtrows(self, date_begin=None, date_end=None):
#            """
#            Returns a list of ddtrows in a certain period
#            """
#            if not date_begin: date_begin = date.today()
#            if not date_end: date_end = date.today()
#            ddts = self.ddts.filter(date_d__gte=date_begin, date_d__lte=date_end)
#            total_rows = []
#            for ddt in ddts:
#                rows = ddt.rows.all()
#                total_rows.extend(rows)
#            return total_rows
#
#    if "ddt" in settings.INSTALLED_APPS:
#        def get_ddtrows_todelivery(self, date_begin=None, date_end=None):
#            """
#            Returns a list of ddtrows to be delivered in a certain period
#            FIXME "to delivery"
#            """
#            if not date_begin: date_begin = date.today()
#            if not date_end: date_end = date.today()
#            ddts = self.ddts.filter(date_d__gte=date_begin, date_d__lte=date_end)
#            total_rows = []
#            for ddt in ddts:
#                rows = ddt.rows.all()
#                total_rows.extend(rows)
#            return total_rows
#
#    if "invoice" in settings.INSTALLED_APPS:
#        def get_invoicerows(self, date_begin=None, date_end=None):
#            """
#            Returns a list of invoicerows in a certain period
#            """
#            if not date_begin: date_begin = date.today()
#            if not date_end: date_end = date.today()
#            invoices = self.owner_invoices.filter(date_i__gte=date_begin, date_i__lte=date_end)
#            total_rows = []
#            for invoice in invoices:
#                rows = invoice.rows.all()
#                total_rows.extend(rows)
#            return total_rows
#
#    if "invoice" in settings.INSTALLED_APPS:
#        def get_total_invoiced(self, year=None):
#            """
#            Returns total income, total fee and total discount of year set
#            """
#            if not year: year = date.today().year
#            totals = self.owner_invoices.filter(
#                date_i__year="%s" % year
#            ).aggregate(
#                Sum('total_income'), Sum('total_fee'),
#                Sum('total_discount'), Sum('total_vat')
#            )
#            totals['year'] = year
#            try:
#                totals['totals__sum'] = (
#                    totals['total_income__sum'] + totals['total_fee__sum']
#                    - totals['total_discount__sum'] + totals['total_vat__sum']
#                    )
#            except:
#                totals['totals__sum'] = None
#            return totals
#
#    if "invoice" in settings.INSTALLED_APPS:
#        def get_year_invoiced(self):
#            """
#            Returns invoice totals of the last 3 years
#            """
#            l = []
#            year = date.today().year
#            for y in [year, year-1, year-2]:
#                l.append(self.get_total_invoiced(y))
#            return l
#
#    if "procedure" in settings.INSTALLED_APPS:
#        def get_procedurerows(self, date_begin=None, date_end=None):
#            """
#            Returns a list of procedurerows in a certain period
#            """
#            if not date_begin: date_begin = date.today()
#            if not date_end: date_end = date.today()
#            procedures = self.procedures.filter(date_p__gte=date_begin, date_p__lte=date_end)
#            total_rows = []
#            for procedure in procedures:
#                rows = procedure.rows.all()
#                total_rows.extend(rows)
#            return total_rows
#
#
#    if "procedure" in settings.INSTALLED_APPS:
#        def get_total_ordered(self, year=None):
#            if not year:
#                year = date.today().year
#            totals = self.owner_procedures.filter(date_p__year="%s" % year).aggregate(Sum('total_income'), Sum('total_fee'), Sum('total_discount'), Sum('total_vat'))
#            totals['year'] = year
#
#            # Aggiungo al totale degli ordini anche un eventuale totale ordinato (o fatturato,
#            # qua e' da capire bene, xke x mentasti il fatturato e' l'ordinato) dell'anno
#            # che rappresenta un'eventuale quantita iniziale settata se si inizia ad usare il
#            # programma a meta' anno
#            #
#            total_invoice = self.total_invoiced.filter(year=year)
#            if total_invoice:
#                total_invoice = total_invoice[0].total
#            else:
#                total_invoice = 0
#
#            try:
#                totals['totals__sum'] = (totals['total_income__sum'] or 0) +\
#                                        (totals['total_fee__sum'] or 0) -\
#                                        (totals['total_discount__sum'] or 0) +\
#                                        (totals['total_vat__sum'] or 0) +\
#                                        total_invoice
#            except Exception, e:
#                totals['totals__sum'] = None
#            return totals
#
#
#    if "procedure" in settings.INSTALLED_APPS:
#        def get_year_ordered(self, num_year=4):
#            """
#            Returns a list of proceure's totals in the last 4 years
#            """
#            l = []
#            first_year = date.today().year - num_year + 1
#            last_year = date.today().year + 1
#
#            for y in reversed(range(first_year, last_year)):
#                l.append(self.get_total_ordered(y))
#            return l
#
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def get_stops(self, date_s = date.today()):
#            """
#            restituisce tutte le sospensioni dell'Anagrafica alla data specificata
#            """
#            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
#            return stop
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def is_stopped(self, date_s = date.today()):
#            """
#            restituisce True o False se il Cliente e' sospeso nel giorno specificato
#            """
#            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
#            if stop: return True
#            return False
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def is_stopped_in(self, date_begin = date.today(), date_end = date.today()):
#            """
#            restituisce True o False se eiste già una sospensione dell'Anagrafica nel periodo specificato.
#            """
#            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
#            if stop: return True
#            return False
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def is_forever_stopped(self, date_s = date.today()):
#            """
#            restituisce True o False se il Cliente e' sospeso definitivamente:
#            e' sospeso definitivamente se non esiste la data di fine
#            """
#            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
#            if stop: return True
#            return False
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def set_start(self, cause, date_end = date.today()):
#            """
#            crea una sospensione con data inizio = NULL e data fine = giorno precedente a quello impostato
#            L'anagrafica è attiva dal giorno impostato
#            """
#            from suspension.models import Suspension
#            suspension = Suspension(organization=self, disable_cause=cause, date_begin=None, date_end=date_end+timedelta(days=-1))
#            suspension.save()
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def set_restart(self, date_end = date.today()):
#            """
#            attiva l'Anagrafica disattivata da una sospensione modificando la
#            data di fine sospensione inserendo il giorno precedente.
#            La data passata deve essere compresa nell'intervallo di sospensione.
#            """
#            suspensions = self.get_stops(date_end)
#            for suspension in suspensions:
#                suspension.date_end = date_end+timedelta(days=-1)
#                suspension.save()
#
#    if "suspension" in settings.INSTALLED_APPS:
#        def set_stop(self, cause, date_begin = date.today(), date_end = None):
#            """
#            disattiva il cliente creando una sospensione dalla data di inizio
#            alla data di fine specificate inserendo una causale.
#            Se la data di fine è NULL l'Anagrafica è sospesa definitivamente.
#            Non verifica se c'è una sovrapposizione di sospensioni
#            CONTROLLI:
#            - La data di inizio deve essere <= alla data di fine
#            - La data di inizio deve essere > della data di ultima consegna (get_last_ddt())
#            """
#            if not cause:
#                return
#            if date_end and date_begin > date_end:
#                return
#            from ddt.utils import get_last_ddt
#            last_ddt = get_last_ddt()
#            if date_begin <= last_ddt.date_d: return
#            from suspension.models import Suspension
#            suspension = Suspension(organization=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
#            suspension.save()
#
#    def is_credit_card_expired(self, date_e = date.today(), days = timedelta(days = 30)):
#        """
#        restituisce True o False se il cliente ha carta di credito sospesa:
#        """
#        pass
#
#    def set_office (self, office):
#        """
#        assegna o modifica ufficio nelle Pratiche Beneficiario e nell'Anagrafica
#        """
#        pass

#class M2MtoVector(models.Model):
#    """
#    Joins customer or supplier organization with vector organization and transport.
#    MODELS:
#    :model: `organization.Organization` as Customer or Supplier organization
#    :model: `organization.Organization` as Vector organization
#    """
#    organization = models.ForeignKey(
#        Organization,
#        verbose_name = _("Organization"),
#        related_name = "vectors"
#    )
#    vector = models.ForeignKey(
#        Organization,
#        verbose_name = _("Vector"),
#        related_name = "organizations"
#    )
#    transport_by = models.CharField(
#        max_length = 3,
#        choices = settings.ORGANIZATION_TRANSPORTTYPE_DEF,
#        db_index = True,
#        verbose_name = _("Transport by"),
#    )
#
#    class Meta:
#        verbose_name = _("Vector")
#        verbose_name_plural = _("Vectors")
#        unique_together = (("organization", "vector"),)
#
#    def __unicode__(self):
#        return u"%s" % self.vector.name
#
#        #def save(self):
#        #if self.organization == self.vector:
#        #raise ValidationError("Vector have to be different from Organization")
#        #if self.organization.organization_type.category == "vec":
#        #raise ValidationError("Select a valid Organization")
#        #if self.vector.organization_type.category != "vec":
#        #raise ValidationError("Select a valid Vector")
#        #super(M2MtoVector, self).save()


##################################
# GRUPPO SOCIETA                 #
##################################
class OrganizationGroup (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Store organization's groups
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("organization group")
        verbose_name_plural = _("organization groups")

    def __unicode__(self):
        return u"%s" % self.name


##################################
# FORMA SOCIETA                  #
##################################
class OrganizationForm (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Store organization's forms
    EXAMPLE: 'SpA', 'Srl', 'Ltd', 'Plc', 'Inc'
    """
    code = models.CharField(
        unique=True,
        max_length=15,
        db_index=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("organization form")
        verbose_name_plural = _("organization forms")
        ordering = ("ordering", "name")

    def __unicode__(self):
        return u"%s" % self.name


####################################
# TIPO ANAGRAFICA                  #
####################################
class OrganizationType (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Stores organization's types
    EXAMPLE: 'end user', 'retailer', 'dealer'
    """
    code = models.CharField(
        unique=True,
        max_length=15,
        db_index=True,
        verbose_name=_("code"),
    )
    category = models.CharField(
        max_length=15,
        choices=settings.ORGANIZATION_ORGANIZATIONTYPE_DEF,
        db_index=True,
        verbose_name=_("category"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("background informations"),
    )

    class Meta:
        verbose_name = _("organization type")
        verbose_name_plural = _("organization types")
        ordering = ('ordering', 'name')

    def __unicode__(self):
        return u"%s" % self.name


#############################
# SETTORE                   #
#############################
class Field (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Stores organization's fields
    EXAMPLE: 'Entertainment', 'Financial'
    """
    code = models.CharField(
        unique=True,
        max_length=15,
        db_index=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("field")
        verbose_name_plural = _("fields")

    def __unicode__(self):
        return u"%s" % self.name


###################################
# CONTATTO RUOLO                  #
###################################
class ContactRole (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Stores contact roles
    EXAMPLE: 'sales manager', 'development manager'
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _("Contact role")
        verbose_name_plural = _("Contact roles")
        ordering = ('name',)

    def __unicode__(self):
        return u"%s" % self.name


###################################
# CONTATTO TITLE                  #
###################################
class ContactTitle (DateModel, UserModel, StatusModel, OrderedModel):

    """
    Stores contact titles
    EXAMPLE: 'mr. mister', 'dr. doctor'
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    abbreviation = models.CharField(
        unique=True,
        max_length=10,
        verbose_name=_("abbreviation"),
    )

    class Meta:
        verbose_name = _("Contact title")
        verbose_name_plural = _("Contact titles")
        ordering = ('name',)

    def __unicode__(self):
        return u"%s" % self.abbreviation


##############################
# CONTATTO                   #
##############################
class ContactManager(models.Manager):

    def get_query_set(self):
        return super(ContactManager, self).get_query_set()


class OwnerContactManager(models.Manager):

    def get_query_set(self):
        qs = super(OwnerContactManager, self).get_query_set()
        return qs.filter(organization__organization_type__category='com', status=1)


class CustomerContactManager(models.Manager):

    def get_query_set(self):
        qs = super(CustomerContactManager, self).get_query_set()
        return qs.filter(organization__organization_type__category='cus', status=1)


class SupplierContactManager(models.Manager):

    def get_query_set(self):
        qs = super(SupplierContactManager, self).get_query_set()
        return qs.filter(organization__organization_type__category='sup', status=1)


class WorkerContactManager(models.Manager):

    def get_query_set(self):
        qs = super(WorkerContactManager, self).get_query_set()
        return qs.filter(
            Q(organization__organization_type__category='com') |
            Q(organization__organization_type__category='sup'),
            status=1
        )


class Contact (DateModel, UserModel, StatusModel, DefaultModel, OrderedModel, ContactMixin):

    """
    Stores contacts
    MODELS:
    :model: `organization.Organization`
    :model: `auth.User`
    :model: `organization.ContactTitle`
    :model: `organization.ContactRole`
    :model: `i18n.Language`
    """

    objects = ContactManager()
    owners = OwnerContactManager()
    customers = CustomerContactManager()
    suppliers = SupplierContactManager()
    workers = WorkerContactManager()

    user = models.OneToOneField(
        User,
        blank=True, null=True,
        db_index=True,
        related_name='contact',
        verbose_name=_("User"),
    )
    organization = models.ForeignKey(
        Organization,
        blank=True, null=True,
        db_index=True,
        related_name='contacts',
        verbose_name=_("Organization"),
    )
    title = models.ForeignKey(
        ContactTitle,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("Title"),
    )
    role = models.ForeignKey(
        ContactRole,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("Role"),
    )
    language = models.ForeignKey(
        Language,
        blank=True, null=True,
        db_index=True,
        verbose_name=_('language'),
        help_text=_('language'),
    )
    code = models.CharField(
        max_length=15,
        db_index=True,
        null=True, blank=True,
        verbose_name=_("code"),
    )
    last_name = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("Last Name"),
    )
    first_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("First Name"),
    )
    phone = models.CharField(
        max_length=20,
        verbose_name=_("Phone"),
        blank=True, null=True
    )
    mobile = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("Mobile"),
    )
    fax = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("Fax"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("Email"),
    )
    note = models.TextField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Note"),
    )
    in_addressbook = models.NullBooleanField(
        blank=True, null=True,
        verbose_name=_("in rubrica"),
        help_text=_("If this number is in the phone address book.")
    )
    ## se i campi successivi sono 'flaggati' il campo 'email' diventa obbligatorio
    send_procedure_by_email = models.BooleanField(
        default=False,
        verbose_name=_("Send Procedure by email"),
    )
    send_ddt_by_email = models.BooleanField(
        default=False,
        verbose_name=_("Send DDT by email"),
    )
    send_invoice_by_email = models.BooleanField(
        default=False,
        verbose_name=_("Send Invoice by email"),
    )
    send_quotation_by_email = models.BooleanField(
        default=False,
        verbose_name=_("Send Quotation by email"),
    )
    photo = models.ImageField(
        upload_to=settings.ORGANIZATION_CONTACT_UPLOAD_PHOTO,
        blank=True, null=True,
        verbose_name=_("Photo"),
    )
    contact_groups = models.ManyToManyField(
        "OrganizationGroup",
        blank=True, null=True,
        db_index=True,
        related_name="contacts",
        verbose_name=_("group"),
    )

    def get_full_name(self):
        """
        returns the contact's full name (last name and first name)
        """
        return "%s %s" % (self.last_name, self.first_name)
    full_name = property(get_full_name)

    class Meta:
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")
        permissions = (
            ("view_contact", "Can view contacts"),
        )
        ordering = ['ordering', 'last_name', 'first_name']

    def save(self, **kw):
        osn = settings.ORGANIZATION_CONTACT_NAME_STYLE
        if osn == "title":
            self.last_name = self.last_name.title()
            self.first_name = self.first_name.title()
        elif osn == "lower":
            self.last_name = self.last_name.lower()
            self.first_name = self.first_name.lower()
        elif osn == "upper":
            self.last_name = self.last_name.upper()
            self.first_name = self.first_name.upper()

        super(Contact, self).save(**kw)

    if "shop" in settings.INSTALLED_APPS:
        """
        Returns basket's status
        """

        def get_basket_status(self):
            baskets = self.basket.all()
            if not baskets:
                return "empty"
            for basket in baskets:
                if basket.product.shop.fill_basket:
                    return "full"
            return "load"

    def get_email(self):
        """Return an email eather in contact or user if any"""
        try:
            return self.email or self.user.email
        except AttributeError:  # no user connected
            pass

    def __unicode__(self):
        txt = u"%s %s %s" % (self.title or '', self.last_name, self.first_name)
        if self.role:
            txt += " (%s)" % self.role
        return txt

##############################
## AGENTI                    #
##############################
#class Agent(DateModel, UserModel, StatusModel, OrderedModel):
#    """
#    Stores Agents
#    MODELS:
#    :model: `auth.User`
#    :model: `organization.Organization`
#    """
#    ## ForeignKey
#    user = models.OneToOneField (
#        User,
#        blank = True, null = True,
#        db_index = True,
#        related_name='agent',
#        verbose_name = _("User"),
#    )
#    organizations = models.ManyToManyField(
#        Organization,
#        through = 'Commision',
#        related_name = "agent",
#        verbose_name = _("Organizations")
#    )
#    ## Campi Modello
#    code = models.CharField(
#        unique = True,
#        max_length = 15,
#        db_index = True,
#        verbose_name = _("code"),
#    )
#    name = models.CharField(
#        max_length = 100,
#        db_index = True,
#        verbose_name = _('Name'),
#    )
#    email = models.EmailField (
#        max_length = 100,
#        blank = True, null = True,
#        verbose_name = _("Email"),
#    )
#    mobile = models.CharField (
#        max_length = 20,
#        blank = True, null = True,
#        verbose_name = _("Mobile"),
#    )
#    area = models.CharField(
#        max_length = 100,
#        blank = True, null = True,
#        verbose_name = _("Area"),
#    )
#    is_area_manager = models.BooleanField(
#        default = False,
#        verbose_name = _("Is Area Manager"),
#    )
#    commision = models.DecimalField(
#        max_digits=4,
#        decimal_places=2,
#        default=0,
#        verbose_name=_("Commision %"),
#        help_text = _("Default commision for the agent. This value will be used if Commision's value is empty")
#    )
#
#    class Meta:
#        verbose_name = _('Agent')
#        verbose_name_plural = _('Agents')
#        permissions = (
#            ("view_agent"    , "Can view agent"),
#            )
#
#    def __unicode__(self):
#        return u'%s %s' % (self.code, self.name)
#
#    def active_organizations(self):
#        """
#        Returns active organizations
#        """
#        return self.organizations.filter(status=1)
#
#    def disactive_organizations(self):
#        """
#        Returns inactive organizations
#        """
#        return self.organizations.filter(status=1)
#
##############################
## PROVVIGIONI               #
##############################
#class Commision(DateModel, UserModel, StatusModel, OrderedModel):
#    """
#    Stores agent's commisions
#    MODELS:
#    :model: `organization.Agent`
#    :model: `organization.Organization`
#    """
#    agent = models.ForeignKey(
#        Agent,
#        related_name = "commisions",
#        verbose_name = _("Agent")
#    )
#    organization = models.ForeignKey(
#        Organization,
#        related_name = "commisions",
#        verbose_name = _("Organization")
#    )
#    commision = models.DecimalField(
#        max_digits=4,
#        decimal_places=2,
#        default=0,
#        verbose_name=_("Commision %"),
#    )
#
#    class Meta:
#        verbose_name = _('Commision')
#        verbose_name_plural = _('Commisions')
#        unique_together = (('agent','organization',),)
#        permissions = (
#            ("view_commision"    , "Can view commision"),
#            )
#
#    def get_commision(self):
#        """
#        Returns agent commision
#        """
#        commision = self.commision
#        if not commision:
#            commision = self.agent.commision
#        return commision
#
#    def __unicode__(self):
#        commision = self.get_commision()
#        return u'%s - %s' % (self.agent, commision)

#class Settings(JSettings):
#    pass

if settings.CMS_ENABLED:
    from .cms import models
